
#pragma ident "@(#)parse.c	1.2 95/03/31"

/*
 * Parse an EPSF file for it's bounding box.  This file was
 * derived directly from code in Adobe's Import example.  See
 * those files for the appropriate disclaimers and copyright
 * information.
 */

#include <stdio.h>
#include <math.h>
#include <X11/Xlib.h>


/*************************************************************
**
** FUNCTION:	EPSFParseFileHeader
**
** DESCRIPTION:	Parse the header of an EPS file for BoundingBox
**
** PARAMETERS:	File pointer and a rect to return the box in.
**
** RETURN:	Whether a bounding box comment was found
**
*************************************************************/

int
EPSFParseFileHeader(
    FILE* fp,
    XRectangle* rect
)
{
#define BBOXLEN 14		/* Length of "%%BoundingBox:" */
#define BEGINDOCUMENTLEN 15	/* Length of "%%BeginDocument" */
#define BEGINBINARYLEN 14	/* Length of "%%BeginBinary:" */
#define BUFLEN 257
    char buf[BUFLEN];
    char buf2[BUFLEN];
    int atend = False;	/* Found an (atend) comment */
    int foundBBox = False;
    float llx, lly, urx, ury;
    int n;
    int nestingLevel = 0;
    unsigned long binaryCount = 0;
    int len;

    while (1) {
	/*
	** If EOF, we're done
	*/
	if (fgets(buf, BUFLEN, fp) == NULL) {
	    goto SUCCESS;
	}	

	len = strlen(buf);

	/*
	** If in binary data ignore line and decrement binary count
	*/
	if (binaryCount != 0) {
	    if (len > binaryCount) binaryCount = 0;
	    else binaryCount -= len;
	    continue;
	}

	/*
	** Line too long?  Give error message and abort
	*/
	if (len == BUFLEN-1 && buf[BUFLEN-1] != '\n') {
	    fprintf(stderr, "Line too long reading file\n");
	    buf[79] = '\0';
	    fprintf(stderr, "Offending line begins\n%s\n", buf);
	    goto FAILURE;
	}

	/*
	** See if we're beginning binary data
	*/
	if (strncmp(buf, "%%BeginBinary:", BEGINBINARYLEN) == 0) {
	    n = sscanf(buf, "%%%%BeginBinary: %lu", &binaryCount);
	    if (n != 1) binaryCount = 0;	/* Malformed comment */

	/*
	** Check for begin/end document comments.  We ignore anything in
	** nested documents
	*/
	}
	else if (strncmp(buf, "%%BeginDocument", BEGINDOCUMENTLEN) == 0) {
	    nestingLevel++;

	}
	else if (strcmp(buf, "%%EndDocument\n") == 0) {
	    nestingLevel--;

	/*
	** If nesting level is 0, look for more comments
	*/
	}
	else if (nestingLevel == 0) {
	    /*
	    ** If we haven't already hit an (atend), the end of the
	    ** comments is a good place to stop looking
	    */
	    if (!atend && (strcmp(buf, "%%EndComments\n") == 0)) {
		goto SUCCESS;
	    }


	    /*
	    ** Check for a bounding box comment
	    */
	    if (strncmp(buf, "%%BoundingBox:", BBOXLEN) == 0) {
		n = sscanf(buf, "%%%%BoundingBox: %f %f %f %f",
			       &llx, &lly, &urx, &ury);

		/*
		** If we didn't find four numbers, check for (atend)
		*/
		if (n != 4) {
		    n = sscanf(buf, "%%%%BoundingBox: %7s", buf2);

		    if (n == 1 && strcmp(buf2, "(atend)") == 0) {
			atend = True;
		    } 
		    else {
			fprintf(stderr, "Malformed %%%%BoundingBox comment\n%s\n", buf);
			goto FAILURE;
		    }
		} 
		else {
		    foundBBox = True;
		    /*
		    ** Convert bounding box to integers, which they should
		    ** be but sometimes aren't
		    */
		    rect->x = (short) floor(llx);
		    rect->y = (short) floor(lly);
		    rect->width  = (unsigned short) (ceil(urx) - floor(llx));
		    rect->height = (unsigned short) (ceil(ury) - floor(lly));
		}
	    }
	}
LOOPEND: ;
    }

SUCCESS:
    /*
    ** Seem to be successful, but make sure we found a bounding box comment.
    */
    if (foundBBox) {
	return True;
    } 
    else {
	fprintf(stderr, "Missing bounding box comment\n");
    }

FAILURE:
    return False;
#undef BUFLEN
#undef BBOXLEN
#undef BEGINDOCUMENTLEN
#undef BEGINBINARYLEN
} /* end EPSFParseFileHeader() */
